import java.util.Iterator;
import java.util.Stack;

/**
 Represents a binary tree in-order iterator.
*/
public class BTreeInOrdIterator<T> implements Iterator<T>
{
/**
 Stack used to contain the CountingNodes during the post-order traversal.
*/
	protected Stack<CountingNode<T>> nodes;


/**
 Nested class used to keep track of how many times a BinaryNode has been popped. Wraps
 a BinaryNode and provides the count.
*/
	private static class CountingNode<T>
	{
		protected CountingNode( BinaryNode<T> n )
		{
			node = n;
			timesPopped = 0;
		}
		protected BinaryNode<T> node;
		protected int timesPopped;
	}
/**
 Iterator requires a binary tree as input.
*/
	public BTreeInOrdIterator( BinaryTree<T> tree )
	{
		// Create stack.
		nodes = new Stack<CountingNode<T>>();
		// Push root node onto stack.
		if( tree.getRoot() != null )
			nodes.push( new CountingNode<T>( tree.getRoot() ) );
	}
/**
 Returns true if the iterator has more elements.
*/
	public boolean hasNext()
	{
		// Check size of stack.
		return nodes.size()>0;
	}
/**
 Returns the next item.
*/
	public T next()
	{
		BinaryNode<T> n;
		CountingNode<T> cn;
		
		if( hasNext() )
		{
			while( true )
			{
				// Pop stack.
				cn = nodes.pop();
				// If node popped for first time...				
				if( ++cn.timesPopped == 1 )
				{
					// Push node back onto stack.
					nodes.push( cn );

					// If left child exists, push it onto stack.					
					if( cn.node.left != null )
					{
						nodes.push( new CountingNode<T>( cn.node.left ) );
					}						
				}
				// If node popped for second time...
				else if( cn.timesPopped == 2 )
				{
					// If right child exists, push it onto stack.
					if( cn.node.right != null )
					{
						nodes.push( new CountingNode<T>( cn.node.right ) );
					}
					// Return node.
					return cn.node.getItem();
				}
			}
		}
		return null;
	}

/**
 Not supported.
*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
}

