import java.util.Iterator;
import java.util.Queue;
import java.util.LinkedList;

/**
 Represents a binary tree level-order iterator.
*/
public class BTreeLevelOrdIterator<T> implements Iterator<T>
{
	private Queue<BinaryNode<T>> qNodes;

/**
 Iterator requires a binary tree as input.
*/
	public BTreeLevelOrdIterator( BinaryTree<T> tree )
	{
		// Create queue.
		qNodes = new LinkedList<BinaryNode<T>>();
		// Put root node into queue.
		if( tree.getRoot() != null )
			qNodes.offer( tree.getRoot() );
	}
/**
 Returns true if the iterator has more elements.
*/
	public boolean hasNext()
	{
		// Check size of stack.
		return qNodes.size()>0;
	}
/**
 Returns the next item.
*/
	public T next()
	{
		BinaryNode<T> n;
		
		if( hasNext() )
		{
			// Remove node from queue.
			n = qNodes.poll();
			if( n.left != null )
				// Put left child onto queue.
				qNodes.offer( n.left );

			if( n.right != null )
				// Put right child onto queue.
				qNodes.offer( n.right );

			// Return the item removed from the queue.	
			return n.getItem();
		}	
		return null;
	}
/**
 Not supported.
*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
}

