import java.util.Iterator;
import java.util.Stack;

/**
 Represents a binary tree pre-order iterator.
*/
public class BTreePreOrdIterator<T> implements Iterator<T>
{
	private Stack<BinaryNode<T>> nodes;

/**
 Iterator requires a binary tree as input.
*/
	public BTreePreOrdIterator( BinaryTree<T> tree )
	{
		// Create stack.
		nodes = new Stack<BinaryNode<T>>();
		// Push root node onto stack.
		if( tree.getRoot() != null )
			nodes.push( tree.getRoot() );
	}
/**
 Returns true if the iterator has more elements.
*/
	public boolean hasNext()
	{
		// Check size of stack.
		return nodes.size()>0;
	}
/**
 Returns the next item.
*/
	public T next()
	{
		BinaryNode<T> n;
		
		if( hasNext() )
		{
			// Pop stack.
			n = nodes.pop();
			if( n.right != null )
				// Push right child onto stack.
				nodes.push( n.right );
			if( n.left != null )
				// Push left child onto stack.
				nodes.push( n.left );
			// Return the popped item.	
			return n.getItem();
		}	
		return null;
	}
/**
 Not supported.
*/
	public void remove()
	{
		throw new UnsupportedOperationException();
	}
}

