/**
 Represents a binary node that contains an item of type T.
*/
public class BinaryNode<T>
{
	private T item;
/**
 Retrieve the left child node.
*/
	public BinaryNode<T> left;
/**
 Retrieve the right child node.
*/
	public BinaryNode<T> right;

/**
 Create a node that contains an item, but no children.
*/	
	public BinaryNode( T item ) { this.item = item; }

/**
 Create a node that contains an item and left and right children nodes..
*/	
	public BinaryNode( T item, BinaryNode<T> left, BinaryNode<T> right )
	{
		this.item = item;
		this.left = left;
		this.right = right;
	}
/**
 Retrieve item.
*/	
	public T getItem() { return item; }	

/**
 Finds the size of the node.
*/	
	public int getSize()
	{
		return getSize( this );
	}

/**
 Finds the height of the node.
*/	
	public int getHeight()
	{
		return getHeight( this );
	}

/**
 Finds the depth of the node.
*/	
	public int getDepth( BinaryNode<T> root )
	{
		return getDepth( root, this );
	}

/**
 Finds the size of the node. 1 + all its descendants.
*/	
	private int getSize( BinaryNode<T> node )
	{
		if( node == null )
			return 0;
		else
			return 1 + getSize( node.left ) + getSize( node.right );
	}
/**
 Finds the height of the node. 1 + max( height(left), height(right) ).
*/	
	private int getHeight( BinaryNode<T> node )
	{
		if( node == null )
			return -1;
		else
			return 1 + Math.max( getHeight( node.left ), getHeight( node.right ) );
	}
/**
 Finds the deptth of the node.
*/	
	private int getDepth( BinaryNode<T> cur, BinaryNode<T> node )
	{
		if( cur == null )
			return -1;
		if( cur == node )
			return 0;
		else
		{
			int left =  getDepth( cur.left,  node );
			int right = getDepth( cur.right, node ) ;
			
			if( left == -1 && right == -1 )
				return -1;
			
			else
				return 1 + Math.max( left, right );
		}
	}
}



